@@ -25,6 +25,7 @@ import math
 import re
 from datetime import date, datetime, timedelta
 from enum import Enum
+from typing import List, Literal, Tuple, Union
 
 from backports.datetime_fromisoformat import MonkeyPatch
 from dateutil.relativedelta import relativedelta
@@ -37,13 +38,13 @@ log = logging.getLogger(__name__)
 
 
 class Frequency(Enum):
-    ANNUAL = 'annual'
-    BI_ANNUAL = 'bi-annual'
-    BI_MONTHLY = 'bi-monthly'
-    QUARTERLY = 'quarterly'
-    MONTHLY = 'monthly'
-    WEEKLY = 'weekly'
-    DAILY = 'daily'
+    ANNUAL = "annual"
+    BI_ANNUAL = "bi-annual"
+    BI_MONTHLY = "bi-monthly"
+    QUARTERLY = "quarterly"
+    MONTHLY = "monthly"
+    WEEKLY = "weekly"
+    DAILY = "daily"
 
     def to_dimension_code(self):
         return {
@@ -68,15 +69,25 @@ period_format_strict_regex_list = [
     (Frequency.BI_ANNUAL, re.compile(r"^(\d{4})-S([1-2])$")),
     (Frequency.BI_MONTHLY, re.compile(r"^(\d{4})-B([1-6])$")),
     (Frequency.QUARTERLY, re.compile(r"^(\d{4})-Q([1-4])$")),
-    (Frequency.MONTHLY, re.compile(r"^(\d{{4}})-({})$".format(month_range_re))),  # YYYY-[01-12]
-    (Frequency.WEEKLY, re.compile(r"^(\d{{4}})-W({})$".format(week_range_re))),  # YYYY-W[01-53]
+    (
+        Frequency.MONTHLY,
+        re.compile(r"^(\d{{4}})-({})$".format(month_range_re)),
+    ),  # YYYY-[01-12]
+    (
+        Frequency.WEEKLY,
+        re.compile(r"^(\d{{4}})-W({})$".format(week_range_re)),
+    ),  # YYYY-W[01-53]
     # YYYY-[01-12]-[01-31]
-    (Frequency.DAILY, re.compile(r"^(\d{{4}})-({})-({})$".format(month_range_re, day_range_re))),
+    (
+        Frequency.DAILY,
+        re.compile(r"^(\d{{4}})-({})-({})$".format(month_range_re, day_range_re)),
+    ),
 ]
 NOT_AVAILABLE = "NA"
 
 
 def iter_periods_dates(frequency: Frequency, min_date: date, max_date: date = None):
+    """Yield dates from `min_date` to `max_date` (if not `None`) by increment of `frequency`."""
     current_date = min_date
     while True:
         yield current_date
@@ -86,7 +97,7 @@ def iter_periods_dates(frequency: Frequency, min_date: date, max_date: date = No
 
 
 def next_period_date(period_date: date, frequency: Frequency):
-    """Return next period date, applying an increment to `period_date` corresponding to `frequency`.
+    """Return next period date, applying an increment to `period_date` of `frequency`.
 
     >>> next_period_date(date(2000, 1, 1), Frequency.ANNUAL)
     datetime.date(2001, 1, 1)
@@ -121,77 +132,35 @@ def next_period_date(period_date: date, frequency: Frequency):
         assert False, frequency
 
 
-def complete_series(periods_dates, values, frequency: Frequency,
-                    all_periods_dates=None, complete_missing_periods=False):
-    """Complete series by adding missing periods and "NA" (not available) values if missing.
+Value = Union[float, Literal["NA"]]
 
-    >>> complete_series([], [], Frequency.ANNUAL)
-    ([], [])
-    >>> complete_series([date(2000, 1, 1)], [10], Frequency.ANNUAL)
-    ([datetime.date(2000, 1, 1)], [10])
-    >>> complete_series([date(2000, 1, 1), date(2001, 1, 1)], [10, 11], Frequency.ANNUAL)
-    ([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1)], [10, 11])
-    >>> complete_series([date(2000, 1, 1), date(2002, 1, 1)], [10, 12], Frequency.ANNUAL, complete_missing_periods=True)
-    ([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1), datetime.date(2002, 1, 1)], [10, 'NA', 12])
-    >>> complete_series([date(2000, 1, 1), date(2002, 1, 1)], [10, 12], Frequency.ANNUAL, all_periods_dates=[date(1998, 1, 1), date(1999, 1, 1), date(2000, 1, 1), date(2002, 1, 1), date(2003, 1, 1)])
-    ([datetime.date(1998, 1, 1), datetime.date(1999, 1, 1), datetime.date(2000, 1, 1), datetime.date(2002, 1, 1), datetime.date(2003, 1, 1)], ['NA', 'NA', 10, 12, 'NA'])
-    >>> complete_series([date(2000, 1, 1), date(2002, 1, 1)], [10, 12], Frequency.ANNUAL, complete_missing_periods=True, all_periods_dates=[date(1998, 1, 1), date(1999, 1, 1), date(2000, 1, 1), date(2002, 1, 1), date(2003, 1, 1)])
-    ([datetime.date(1998, 1, 1), datetime.date(1999, 1, 1), datetime.date(2000, 1, 1), datetime.date(2001, 1, 1), datetime.date(2002, 1, 1), datetime.date(2003, 1, 1)], ['NA', 'NA', 10, 'NA', 12, 'NA'])
+
+def align_series_list(series_list: List[Tuple[List[date], List[Value]]]):
+    """Align many series.
+
+    Many series are aligned if they share a common list of periods.
+
+    Alignment is done by doing the union of the periods of all series, sorting them,
+    then iterating over series adding NA values when necessary.
     """
-    assert len(periods_dates) == len(values), (periods_dates, values)
-    if not periods_dates:
-        return ([], [])
-
-    completed_periods_dates = []
-    completed_values = []
-    value_by_period_date = dict(zip(periods_dates, values))
-    for expected_period_date in iter_periods_dates(
-            frequency,
-            min_date=min(all_periods_dates) if all_periods_dates is not None else periods_dates[0],
-            max_date=max(all_periods_dates) if all_periods_dates is not None else periods_dates[-1]):
-        value = value_by_period_date.get(expected_period_date)
-        if value is not None or complete_missing_periods or expected_period_date in all_periods_dates:
-            completed_periods_dates.append(expected_period_date)
-            completed_values.append(value if value is not None else NOT_AVAILABLE)
-    return (completed_periods_dates, completed_values)
-
-
-def complete_series_list(periods_dates_and_values_list, frequency: Frequency,
-                         align_periods=False, complete_missing_periods=False):
-    """Complete many series.
-
-    See also: `complete_series`.
-
-    >>> complete_series_list([], Frequency.ANNUAL)
-    []
-
-    # Annual:
-    >>> complete_series_list([([date(2000, 1, 1), date(2001, 1, 1)], [10, 11]), ([date(2001, 1, 1), date(2002, 1, 1)], [21, 22])], Frequency.ANNUAL)
-    [([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1)], [10, 11]), ([datetime.date(2001, 1, 1), datetime.date(2002, 1, 1)], [21, 22])]
-    >>> complete_series_list([([date(2000, 1, 1), date(2001, 1, 1)], [10, 11]), ([date(2001, 1, 1), date(2002, 1, 1)], [21, 22])], Frequency.ANNUAL, align_periods=True)
-    [([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1), datetime.date(2002, 1, 1)], [10, 11, 'NA']), ([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1), datetime.date(2002, 1, 1)], ['NA', 21, 22])]
-    >>> complete_series_list([([date(2000, 1, 1), date(2001, 1, 1)], [10, 11]), ([date(2001, 1, 1), date(2003, 1, 1)], [21, 22])], Frequency.ANNUAL, complete_missing_periods=True)
-    [([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1)], [10, 11]), ([datetime.date(2001, 1, 1), datetime.date(2002, 1, 1), datetime.date(2003, 1, 1)], [21, 'NA', 22])]
-    >>> complete_series_list([([date(2000, 1, 1), date(2001, 1, 1)], [10, 11]), ([date(2001, 1, 1), date(2003, 1, 1)], [21, 22])], Frequency.ANNUAL, align_periods=True, complete_missing_periods=True)
-    [([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1), datetime.date(2002, 1, 1), datetime.date(2003, 1, 1)], [10, 11, 'NA', 'NA']), ([datetime.date(2000, 1, 1), datetime.date(2001, 1, 1), datetime.date(2002, 1, 1), datetime.date(2003, 1, 1)], ['NA', 21, 'NA', 22])]
-
-    # Monthly:
-    >>> complete_series_list([([date(2000, 1, 1), date(2000, 2, 1)], [10, 11]), ([date(2000, 2, 1), date(2000, 3, 1)], [21, 22])], Frequency.MONTHLY)
-    [([datetime.date(2000, 1, 1), datetime.date(2000, 2, 1)], [10, 11]), ([datetime.date(2000, 2, 1), datetime.date(2000, 3, 1)], [21, 22])]
-    >>> complete_series_list([([date(2000, 1, 1), date(2000, 2, 1)], [10, 11]), ([date(2000, 2, 1), date(2000, 3, 1)], [21, 22])], Frequency.MONTHLY, align_periods=True)
-    [([datetime.date(2000, 1, 1), datetime.date(2000, 2, 1), datetime.date(2000, 3, 1)], [10, 11, 'NA']), ([datetime.date(2000, 1, 1), datetime.date(2000, 2, 1), datetime.date(2000, 3, 1)], ['NA', 21, 22])]
-    """
-    if not periods_dates_and_values_list:
+    if not series_list:
         return []
+    if len(series_list) == 1:
+        return series_list
 
-    all_periods_dates = set(mapcat(lambda pair: pair[0], periods_dates_and_values_list)) \
-        if align_periods \
-        else None
-    return [
-        complete_series(periods_dates, values, frequency, all_periods_dates=all_periods_dates,
-                        complete_missing_periods=complete_missing_periods)
-        for periods_dates, values in periods_dates_and_values_list
-    ]
+    # Build all_periods: a set containing all dates of all series given in series_list
+    all_periods = sorted(set(mapcat(lambda pair: pair[0], series_list)))
+
+    # Build aligned_series_list
+    aligned_series_list = []
+    for periods, values in series_list:
+        value_by_period = dict(zip(periods, values))
+        aligned_values = [
+            value_by_period.get(period) or NOT_AVAILABLE for period in all_periods
+        ]
+        aligned_series_list.append((all_periods, aligned_values))
+
+    return aligned_series_list
 
 
 def detect_frequency(period1, period2):
@@ -285,13 +254,15 @@ def detect_frequency(period1, period2):
             else None
         )
 
-
     period1_format, normalize_period = detect_period_format(period1)
     period2_format, _ = detect_period_format(period2)
 
     if period1_format != period2_format:
-        log.error("The 2 periods do not have the same format {!r}".format(
-            ((period1, period1_format), (period2, period2_format))))
+        log.error(
+            "The 2 periods do not have the same format {!r}".format(
+                ((period1, period1_format), (period2, period2_format))
+            )
+        )
         return (None, None)
 
     elif period1_format == Frequency.DAILY:
@@ -302,17 +273,35 @@ def detect_frequency(period1, period2):
         if date_diff.days == 1:
             return (Frequency.DAILY, None)
         elif date_diff.days == 7:
-            return (Frequency.WEEKLY, make_normalize_period(Frequency.WEEKLY, date_1, date_2))
+            return (
+                Frequency.WEEKLY,
+                make_normalize_period(Frequency.WEEKLY, date_1, date_2),
+            )
         elif date_diff.months == 1:
-            return (Frequency.MONTHLY, make_normalize_period(Frequency.MONTHLY, date_1, date_2))
+            return (
+                Frequency.MONTHLY,
+                make_normalize_period(Frequency.MONTHLY, date_1, date_2),
+            )
         elif date_diff.months == 2:
-            return (Frequency.BI_MONTHLY, make_normalize_period(Frequency.BI_MONTHLY, date_1, date_2))
+            return (
+                Frequency.BI_MONTHLY,
+                make_normalize_period(Frequency.BI_MONTHLY, date_1, date_2),
+            )
         elif date_diff.months == 3:
-            return (Frequency.QUARTERLY, make_normalize_period(Frequency.QUARTERLY, date_1, date_2))
+            return (
+                Frequency.QUARTERLY,
+                make_normalize_period(Frequency.QUARTERLY, date_1, date_2),
+            )
         elif date_diff.months == 6:
-            return (Frequency.BI_ANNUAL, make_normalize_period(Frequency.BI_ANNUAL, date_1, date_2))
+            return (
+                Frequency.BI_ANNUAL,
+                make_normalize_period(Frequency.BI_ANNUAL, date_1, date_2),
+            )
         elif date_diff.years == 1:
-            return (Frequency.ANNUAL, make_normalize_period(Frequency.ANNUAL, date_1, date_2))
+            return (
+                Frequency.ANNUAL,
+                make_normalize_period(Frequency.ANNUAL, date_1, date_2),
+            )
         else:
             # No specific interval detected, keep daily.
             return (Frequency.DAILY, None)
@@ -436,6 +425,7 @@ def detect_period_format(period):
         def _normalize_period(period):
             match = regex.match(period)
             return format_match(match) if match is not None else period
+
         return _normalize_period
 
     for period_format, regex, format_match in [
@@ -466,7 +456,9 @@ def detect_period_format(period):
         ),
         (
             Frequency.DAILY,
-            re.compile(r"^(\d{4})[-\s]*(0[1-9]|1[0-2])[-\s]*(0[1-9]|[1-9]|[1-2][0-9]|3[0-1])$"),
+            re.compile(
+                r"^(\d{4})[-\s]*(0[1-9]|1[0-2])[-\s]*(0[1-9]|[1-9]|[1-2][0-9]|3[0-1])$"
+            ),
             lambda match: "{0}-{1:>02}-{1:>02}".format(*match.groups()),
         ),
     ]:
@@ -520,7 +512,9 @@ def detect_period_format_strict(period):
     >>> detect_period_format_strict("ABCDE")
     >>> detect_period_format_strict("2014Z01")
     """
-    assert isinstance(period, str), "{!r} should by a string; got {}".format(period, type(period))
+    assert isinstance(period, str), "{!r} should by a string; got {}".format(
+        period, type(period)
+    )
     for period_format, regex in period_format_strict_regex_list:
         if regex.match(period):
             return period_format
@@ -563,7 +557,7 @@ def iter_tsv_rows(tsv_io):
     [['period', 'value'], ['2018', 'NaN']]
     """
     for line in tsv_io:
-        line = line.strip('\n')
+        line = line.strip("\n")
         if not line:
             continue
         cells = line.split("\t")
@@ -636,13 +630,13 @@ def normalize_observations(observations):
         frequency, normalize_period = detect_frequency(period1, period2)
 
     def normalize_row(row):
-        period = normalize_period(row[0]) \
-            if normalize_period is not None \
-            else row[0]
+        period = normalize_period(row[0]) if normalize_period is not None else row[0]
         value = value_to_float(get(1, row, None))
         attributes = [
             get(index, row, None) or ""
-            for index, column_name in enumerate(header[2:], start=2)  # Skip period and value, already processed.
+            for index, column_name in enumerate(
+                header[2:], start=2
+            )  # Skip period and value, already processed.
         ]
         return [period, value] + attributes
 
