@@ -51,22 +51,32 @@ def generate_series_name(dimensions, dimensions_codes_order, dimensions_values_l
     >>> generate_series_name({'FREQ': 'A', 'INDICATOR': 'Z'}, ['FREQ', 'INDICATOR'], {'FREQ': {'A': 'Annual'}, 'INDICATOR': {'X': 'Files', 'Y': 'Files'}})
     'Annual – Z'
     """
+
     def get_dimension_value_label(dimension_code):
         dimension_value_code = dimensions.get(dimension_code)
         if dimension_value_code is None:
-            log.error("Missing dimension value code in Solr series for dimension_code {}: {}".format(
-                dimension_code, dimensions))
+            log.error(
+                "Missing dimension value code in Solr series for dimension_code {}: {}".format(
+                    dimension_code, dimensions
+                )
+            )
             return "{} (MISSING CODE)".format(dimension_code)
         dimension_values_labels = dimensions_values_labels.get(dimension_code, {})
         dimension_value_label = dimension_values_labels.get(dimension_value_code)
         if dimension_value_label is None:
             dimension_value_label = dimension_value_code
         else:
-            if len([
-                v for v in dimension_values_labels.values()
-                if v == dimension_value_label
-            ]) > 1:
-                dimension_value_label += ' ({})'.format(dimension_value_code)
+            if (
+                len(
+                    [
+                        v
+                        for v in dimension_values_labels.values()
+                        if v == dimension_value_label
+                    ]
+                )
+                > 1
+            ):
+                dimension_value_label += " ({})".format(dimension_value_code)
         return dimension_value_label
 
     series_name = " – ".join(map(get_dimension_value_label, dimensions_codes_order))
@@ -86,5 +96,7 @@ def series_ids_to_tree(series_ids):
     """
     tree = {}
     for (provider_code, dataset_code, series_code) in series_ids:
-        tree.setdefault(provider_code, {}).setdefault(dataset_code, []).append(series_code)
+        tree.setdefault(provider_code, {}).setdefault(dataset_code, []).append(
+            series_code
+        )
     return tree
